@@ -1,10 +1,14 @@
+"""Defines the Data Classes used."""
 from datetime import datetime as dt
-from dateutil import tz
+import datetime
+import pytz
+
 from weatherbitpypi.const import SUPPORTED_LANGUAGES
 import json
 import os
+import logging
 
-"""Defines the Data Classes used."""
+_LOGGER = logging.getLogger(__name__)
 
 class CurrentData:
     """A representation of Current Weather Data."""
@@ -199,13 +203,13 @@ class CurrentData:
 
     @property
     def sunrise(self) -> str:
-        """Sunrise time (HH:MM)."""
-        return self._sunrise
+        """Sunrise time (HH:MM) Local Timezone."""
+        return get_timezone_time(self._sunrise, self._timezone, True)
 
     @property
     def sunset(self) -> str:
-        """Suntime time (HH:MM)."""
-        return self._sunset
+        """Suntime time (HH:MM) Local Timezone."""
+        return get_timezone_time(self._sunset, self._timezone, False)
 
     @property
     def datetime(self) -> str:
@@ -223,14 +227,8 @@ class CurrentData:
 
     @property
     def obs_time_local(self) -> str:
-        """Observation Time at Location."""
-        from_zone = tz.gettz("UTC")
-        to_zone = tz.gettz(self.timezone)
-        obs_time = dt.strptime(self.ob_time, "%Y-%m-%d %H:%M")
-        obs_day = obs_time.replace(tzinfo=from_zone)
-        obs_local = obs_day.astimezone(to_zone)
-
-        return obs_local.strftime("%Y-%m-%d %H:%M")
+        """Observation Time at Location (Local Timezone)."""
+        return get_timezone_date(self.ob_time, self._timezone, "%Y-%m-%d %H:%M")
 
     @property
     def beaufort_value(self) -> int:
@@ -273,24 +271,7 @@ class CurrentData:
         return get_localized_text(self._language, self.beaufort_value, "beaufort")
 
 
-def get_localized_text(language, value, index):
-    """Read the localized string from the Language file."""
-    if language not in SUPPORTED_LANGUAGES:
-        filename = f"/translations/en.json"
-    else:
-        filename = f"/translations/{language}.json"
-
-    # Build filepath
-    cwd = __file__
-    path_index = cwd.rfind("/")
-    top_path = cwd[0:path_index]
-    filepath = f"{top_path}{filename}"
-
-    # Return Value from language string
-    with open(filepath) as json_file:
-        data = json.load(json_file)
-        return data[index][str(value)]
-
+     
 class ForecastDailyData:
     """A representation of Daily Forecast Weather Data."""
 
@@ -340,11 +321,8 @@ class ForecastDailyData:
     @property
     def timestamp(self) -> dt:
         """Date the forecast is valid for (YYYY-MM-DD HH:MM:ss)"""
-        from_zone = tz.gettz("UTC")
-        to_zone = tz.gettz(self._timezone)
-        ts = dt.fromtimestamp(self._ts)
-        date_notz = ts.replace(tzinfo=from_zone)
-        return date_notz.astimezone(to_zone)
+        ts = dt.fromtimestamp(self._ts).strftime("%Y-%m-%d %H:%M:%S")
+        return get_timezone_date(ts, self._timezone, "%Y-%m-%d %H:%M:%S")
 
     @property
     def ts(self) -> float:
@@ -673,3 +651,61 @@ class ForecastHourlyData:
     def timezone(self):
         """Local IANA Timezone."""
         return self._timezone
+
+############################
+# Helper Functions below
+############################
+
+def get_localized_text(language, value, index):
+    """Read the localized string from the Language file."""
+    if language not in SUPPORTED_LANGUAGES:
+        filename = f"/translations/en.json"
+    else:
+        filename = f"/translations/{language}.json"
+
+    # Build filepath
+    cwd = __file__
+    path_index = cwd.rfind("/")
+    top_path = cwd[0:path_index]
+    filepath = f"{top_path}{filename}"
+
+    # Return Value from language string
+    with open(filepath) as json_file:
+        data = json.load(json_file)
+        return data[index][str(value)]
+
+def get_timezone_time(value, timezone_local, sunrise):
+    """Returns %H:%M in local timezone from a UTC time."""
+    val_arr = value.split(":")
+    tz_local = pytz.timezone(timezone_local)
+    tz_utc = pytz.utc
+    hour = int(val_arr[0])
+    minute = int(val_arr[1])
+    day_part = dt.today()
+    year = day_part.year
+    month = day_part.month
+    day = day_part.day
+    
+    val_utc = dt(year, month, day, hour, minute, 0, tzinfo=tz_utc)
+    val_local = val_utc.astimezone(tz_local)
+    # Time has passed, move to next day
+    if val_local < dt.now(tz_local):
+        val_local = val_local + datetime.timedelta(days=1)
+    return val_local.strftime("%Y-%m-%d %H:%M")
+
+def get_timezone_date(value, timezone_local, time_format):
+    """Returns %Y-%m-%d %H:%M in local timezone from a UTC datetime."""
+    tz_local = pytz.timezone(timezone_local)
+    tz_utc = pytz.utc
+    try:
+        val_date = dt.strptime(value, time_format)
+    except ValueError:
+       _LOGGER.debug(f"Date in Wrong Format: {value}")
+       return None
+    year = val_date.year
+    month = val_date.month
+    day = val_date.day
+    hour = val_date.hour
+    minute = val_date.minute
+    val_utc = dt(year, month, day, hour, minute, 0, tzinfo=tz_utc)
+    return val_utc.astimezone(tz_local).strftime(time_format)
